@@ -561,9 +561,14 @@ gui_mch_add_menu_item(GuiMenu *menu, GuiMenu *parent, int idx)
     assert(menu != NULL);
     assert(parent != NULL);
 
+    /* Don't add menu separator */
+    if (gui_is_separator(menu->name))
+	return;
+
     if (parent->menuItemPtr == NULL)
 	return;
 
+    /* TODO: use menu->mnemonic and menu->actext */
     menutext = (struct IntuiText *) malloc(sizeof(struct IntuiText));
 
     SetAttrib(menutext,FrontPen,3);
@@ -779,6 +784,27 @@ gui_mch_exit(int returnCode)
     }
 }
 
+/*
+ * Get the position of the top left corner of the window.
+ */
+    void
+gui_mch_get_winpos(int *x, int *y)
+{
+    /* TODO */
+    *x = -1;
+    *y = -1;
+}
+
+/*
+ * Set the position of the top left corner of the window to the given
+ * coordinates.
+ */
+    void
+gui_mch_set_winpos(int x, int y)
+{
+    /* TODO */
+}
+
     void
 gui_mch_set_winsize(int width, int height, int min_width, int min_height,
 		    int base_width, int base_height)
@@ -854,16 +880,19 @@ gui_mch_set_font(GuiFont font)
     /*D("gui_mch_set_font");*/
 }
 
+#if 0 /* not used */
     int
-gui_mch_same_font( GuiFont f1, GuiFont f2)
+gui_mch_same_font(GuiFont f1, GuiFont f2)
 {
     D("gui_mch_same_font");
 }
+#endif
 
     void
-gui_mch_free_font( GuiFont font)
+gui_mch_free_font(GuiFont font)
 {
-    D("gui_mch_free_font");
+    if (font)
+	D("gui_mch_free_font");
 }
 
 #define RGB(a,b,c) ((a&&0xff)*0x10000+(b*0xff)*0x100+(c&0xff))
@@ -1228,6 +1257,13 @@ mch_setmouse(int  on)
  * Get current y mouse coordinate in text window.
  * Return -1 when unknown.
  */
+    int
+gui_mch_get_mouse_x()
+{
+    /* TODO */
+    return -1;
+}
+
     int
 gui_mch_get_mouse_y()
 {
@@ -1243,6 +1279,12 @@ gui_mch_setmouse(x, y)
     /* TODO */
 }
 
+    void
+gui_mch_show_popupmenu(GuiMenu *menu)
+{
+    /* TODO */
+}
+
     void
 clip_mch_lose_selection(void)
 {
@@ -1287,5 +1329,4 @@ gui_mch_get_rgb(GuiColor pixel)
     return retval;
 }
 
-
 #endif /* USE_AMIGA_GUI*/
