@@ -481,6 +481,7 @@ RefsReceived(BMessage *m, bool changedir)
     if (VIsual_active)
     {
 	end_visual_mode();
+	VIsual_reselect = FALSE;
 	update_curbuf(NOT_VALID);	/* delete the inversion */
     }
 
@@ -1961,6 +1962,27 @@ gui_mch_exit(int vim_exitcode)
     /* If we are somehow still here, let mch_windexit() handle things. */
 }
 
+/*
+ * Get the position of the top left corner of the window.
+ */
+    void
+gui_mch_get_winpos(int *x, int *y)
+{
+    /* TODO */
+    *x = -1;
+    *y = -1;
+}
+
+/*
+ * Set the position of the top left corner of the window to the given
+ * coordinates.
+ */
+    void
+gui_mch_set_winpos(int x, int y)
+{
+    /* TODO */
+}
+
 /*
  * Set the size of the window to the given width and height in pixels.
  */
@@ -2353,6 +2375,7 @@ gui_mch_set_font(
     }
 }
 
+#if 0 /* not used */
 /*
  * Return TRUE if the two fonts given are equivalent.
  */
@@ -2368,6 +2391,7 @@ gui_mch_same_font(
 	    (vf1->FamilyAndStyle() == vf2->FamilyAndStyle() &&
 	     vf1->Size() == vf2->Size());
 }
+#endif
 
 /* XXX TODO This is apparently never called... */
     void
@@ -2917,17 +2941,14 @@ gui_mch_add_menu(
     GuiMenu	*parent
     int		idx)
 {
-    char_u	*menu_name;
-    int		mnemonic = 0;
-
     if (!gui_menubar_menu(menu->name)
 	    || (parent != NULL && parent->submenu_id == NULL))
 	return;
 
-    menu_name = gui_menu_text(menu->name, &mnemonic);
-    if (menu_name != NULL && gui.vimWindow->Lock())
+    /* TODO: use menu->mnemonic and menu->actext */
+    if (gui.vimWindow->Lock())
     {
-	BMenu *bmenu = new BMenu((char *)menu_name);
+	BMenu *bmenu = new BMenu((char *)menu->dname);
 	BMenuItem *bmenuitem = new BMenuItem(bmenu);
 
 	menu->id = bmenuitem;
@@ -2962,7 +2983,6 @@ gui_mch_add_menu(
 
 	gui.vimWindow->Unlock();
     }
-    vim_free(menu_name)
 }
 
     void
@@ -2990,23 +3010,25 @@ gui_mch_add_menu_item(
     GuiMenu	*parent
     int		idx)
 {
-    char_u	*menu_name;
     int		mnemonic = 0;
 
     if (parent->submenu_id == NULL)
 	return;
 
-    menu_name = gui_menu_text(menu->name, &mnemonic);
-    if (menu_name != NULL && gui.vimWindow->Lock())
+    /* Don't add menu separator */
+    if (gui_is_separator(menu->name))
+	return;
+
+    /* TODO: use menu->mnemonic and menu->actext */
+    if (gui.vimWindow->Lock())
     {
-	BMenuItem *item = new BMenuItem((char *)menu_name, MenuMessage(menu));
+	BMenuItem *item = new BMenuItem((char *)menu->dname, MenuMessage(menu));
 	item->SetTarget(gui.vimTextArea);
 	parent->submenu_id->AddItem(item);
 	menu->id = item;
 	menu->submenu_id = NULL;
 	gui.vimWindow->Unlock();
     }
-    vim_free(menu_name);
 }
 
 /*
@@ -3230,5 +3252,18 @@ gui_mch_get_rgb(
     return retval;
 }
 
+    void
+gui_mch_setmouse(int x, int y)
+{
+    /* TODO */
+}
+
+    void
+gui_mch_show_popupmenu(GuiMenu *menu)
+{
+    /* TODO */
+}
+
+
 } /* extern "C" */
 
