@@ -167,13 +167,13 @@ gui_mac_get_menu_item_index (menu, parent)
 {
 	GuiMenu *brothers  = parent->children;
 	short	 itemIndex =1;
-	
+
 	for (; brothers != NULL; brothers = brothers->next, itemIndex++)
 		if (brothers == menu)
 			break;
 
 /*  if (brothers == NULL)
-        TODO: flag an error       */
+		TODO: flag an error		  */
 
 	return (itemIndex);
 }
@@ -270,8 +270,8 @@ gui_mac_focus_change(event)
 gui_mac_handle_menu(menuChoice)
 	long menuChoice;
 {
-	short	 menu        = HiWord(menuChoice);
-	short	 item        = LoWord(menuChoice);
+	short	 menu		 = HiWord(menuChoice);
+	short	 item		 = LoWord(menuChoice);
 	GuiMenu  *theVimMenu = gui.root_menu;
 
 	if (menu == 256)  /* TODO: use constant or gui.xyz */
@@ -433,7 +433,7 @@ gui_mch_prepare(argc, argv)
     HGetVol (volName, &applVRefNum, &applDirID);
     FSMakeFSSpec (applVRefNum, applDirID, "\p", &applDir);
     GetFullPathFromFSSpec (temp, applDir);
-    exe_name = FullName_save((char_u *)temp, FALSE); 
+    exe_name = FullName_save((char_u *)temp, FALSE);
 #endif
 
 #ifdef USE_VIM_CREATOR_ID
@@ -511,6 +511,27 @@ gui_mch_exit()
 	/* Exit to shell? */
 }
 
+/*
+ * Get the position of the top left corner of the window.
+ */
+    void
+gui_mch_get_winpos(int *x, int *y)
+{
+    /* TODO */
+    *x = -1;
+    *y = -1;
+}
+
+/*
+ * Set the position of the top left corner of the window to the given
+ * coordinates.
+ */
+    void
+gui_mch_set_winpos(int x, int y)
+{
+    /* TODO */
+}
+
 	void
 gui_mch_set_winsize(width, height, min_width, min_height,
 					base_width, base_height)
@@ -601,24 +622,25 @@ gui_mac_find_font (font_name)
     short       font_id;
     short       size;
     GuiFont     font;
-    
-    for (p = font_name; ((*p != 0) && (*p != ':')); p++);
-    
+
+    for (p = font_name; ((*p != 0) && (*p != ':')); p++)
+		;
+
 		c = *p;
 		*p = 0;
 		STRCPY(&pFontName[1], font_name);
 		pFontName[0] = STRLEN(font_name);
 		*p = c;
-    
+
     GetFNum (pFontName, &font_id);
-    
+
     if (font_id == 0)
 		{
 			/* Oups, the system font was it the one the user want */
 
 			GetFontName (0, systemFontname);
 			if (!EqualString(pFontName, systemFontname, false, false))
-            return (GuiFont) 0;
+			return (GuiFont) 0;
 		}
 		if (*p == ':')
 		{
@@ -631,20 +653,20 @@ gui_mac_find_font (font_name)
 					case 'h':
 						size = points_to_pixels(p, &p, TRUE);
 						break;
-                /*
-                 * TODO: Maybe accept width and styles
-                 */
+				/*
+				 * TODO: Maybe accept width and styles
+				 */
 				}
 				while (*p == ':')
 					p++;
 			}
-    }
+	}
 
-    if (size < 1)
-        size = 1;   /* Avoid having a size of 0 with system font */
+	if (size < 1)
+		size = 1;	/* Avoid having a size of 0 with system font */
 
 	font = (size << 16) + ((long) font_id & 0xFFFF);
-	
+
     return font;
 }
 
@@ -662,7 +684,7 @@ gui_mch_init_font(font_name)
     FontInfo    font_info;
     short       font_id;
     GuiFont     font;
-     
+
 	if (font_name == NULL)
 	{
 		/* First try to get the suggested font */
@@ -673,17 +695,17 @@ gui_mch_init_font(font_name)
 			/* Then pickup the standard application font */
 			font_id = GetAppFont();
 		}
-        font = (suggestedSize << 16) + ((long) font_id & 0xFFFF);
+		font = (suggestedSize << 16) + ((long) font_id & 0xFFFF);
 	}
 	else
 	{
-        font = gui_mac_find_font (font_name);
-        
-        if (font == (GuiFont) 0)
-            return FAIL;
+		font = gui_mac_find_font (font_name);
+
+		if (font == (GuiFont) 0)
+			return FAIL;
 	}
 	gui.norm_font = font;
-	
+
     TextSize (font >> 16);
 	TextFont (font & 0xFFFF);
 
@@ -737,6 +759,7 @@ gui_mch_set_font(font)
 	TextFont (font & 0xFFFF);
 }
 
+#if 0 /* not used */
 /*
  * Return TRUE if the two fonts given are equivalent.
  */
@@ -747,6 +770,7 @@ gui_mch_same_font(f1, f2)
 {
 	return f1 == f2;
 }
+#endif
 
 /*
  * If a font is not going to be used, free its structure.
@@ -756,7 +780,8 @@ gui_mch_free_font(font)
 	GuiFont		font;
 {
 	/*
-     * Nothing to do in the current implementation, since 
+	 * Free font when "font" is not 0.
+     * Nothing to do in the current implementation, since
      * nothing is allocated for each font used.
      */
 }
@@ -1068,7 +1093,7 @@ gui_mac_do_key(EventRecord *theEvent)
 	key_sym = ((theEvent->message & keyCodeMask) >> 8);
 	string[0] = theEvent->message & charCodeMask;
 	num = 1;
-	
+
 	if (theEvent->modifiers & controlKey)
 		if (string[0] == Ctrl('C'))
 			got_int = TRUE;
@@ -1319,11 +1344,11 @@ gui_mac_handle_event (event)
 				gui_mac_focus_change(event);
 			}
 			break;
-		
+
 #ifdef USE_AEVENT
-        case (kHighLevelEvent):
-            (void) AEProcessAppleEvent(event); /* TODO: Error Handling */
-            break;
+		case (kHighLevelEvent):
+			(void) AEProcessAppleEvent(event); /* TODO: Error Handling */
+			break;
 #endif
 	}
 }
@@ -1738,12 +1763,12 @@ gui_mch_add_menu(menu, parent, idx)
 {
 	/*
 	 * TODO: Simplify handling of index, handle and id of menu and submenu
+	 * TODO: use menu->mnemonic and menu->actext
 	 */
 	static long	 next_avail_id = 128;
 	char_u		*name;
 	long		len;
 	short		index;
-	int			mnemonic;
 
     if (!gui_menubar_menu(menu->name)
 			|| (parent != NULL && parent->submenu_id == 0))
@@ -1752,9 +1777,11 @@ gui_mch_add_menu(menu, parent, idx)
 	if (next_avail_id == 1024)
 		return;
 
-	name = gui_menu_text(menu->name, &mnemonic);
-	len = strlen((char *)menu->name);
-	mch_memmove (name + 1, name, len);
+	len = STRLEN(menu->dname);
+	name = alloc(len + 1);
+	if (name == NULL)
+		return;
+	mch_memmove(name + 1, menu->dname, len);
 	name[0] = len;
 
 	menu->submenu_id = next_avail_id;
@@ -1782,7 +1809,7 @@ gui_mch_add_menu(menu, parent, idx)
 		InsertMenu(menu->submenu_handle, -1);
 	}
 
-	vim_free (name);
+	vim_free(name);
 	next_avail_id++;
 
 #if 0
@@ -1801,16 +1828,19 @@ gui_mch_add_menu_item(menu, parent, idx)
 {
 	char_u		*name;
 	long		len;
-	int			mnemonic = 0;
 
 	if (parent->submenu_id == 0)
 		return;
 
-	name = gui_menu_text(menu->name, &mnemonic);
+	/* Don't add menu separator */
+	if (gui_is_separator(menu->name))
+		return;
+
+	len = STRLEN(menu->dname);
+	name = alloc(len + 1);
 	if (name == NULL)
 		return;
-	len = strlen((char *)name);
-	mch_memmove(name + 1, name, len);
+	mch_memmove(name + 1, menu->dname, len);
 	name[0] = len;
 
 	menu->menu_id = parent->submenu_id;
@@ -1823,6 +1853,8 @@ gui_mch_add_menu_item(menu, parent, idx)
 #if 0
 	DrawMenuBar();
 #endif
+
+	vim_free(name);
 }
 
     void
@@ -2097,12 +2129,12 @@ OSErr HandleUnusedParms (AppleEvent *theAEvent)
 	long		actualSize;
 	DescType	dummyType;
 	AEKeyword	missedKeyword;
-		
+
 /* Get the "missed keyword" attribute from the AppleEvent. */
 	error = AEGetAttributePtr(theAEvent, keyMissedKeywordAttr,
-                              typeKeyword, &dummyType,
+							  typeKeyword, &dummyType,
 							  (Ptr)&missedKeyword, sizeof(missedKeyword),
-                              &actualSize);
+							  &actualSize);
 
 /* If the descriptor isn't found, then we got the required parameters. */
 	if (error == errAEDescNotFound)
@@ -2120,7 +2152,7 @@ OSErr HandleUnusedParms (AppleEvent *theAEvent)
 
 /*
  * Handle the ODoc AppleEvent
- * 
+ *
  * Deals with all files dragged to the application icon.
  *
  */
@@ -2138,7 +2170,7 @@ pascal OSErr HandleODocAE (AppleEvent *theAEvent, AppleEvent *theReply, long ref
 	long		numFiles, fileCount;
     char_u      **fnames;
     char_u      fname[256];
-    
+
 /* the direct object parameter is the list of aliases to files (one or more) */
 	error = AEGetParamDesc(theAEvent, keyDirectObject, typeAEList, &theList);
 	if (error)
@@ -2163,11 +2195,12 @@ pascal OSErr HandleODocAE (AppleEvent *theAEvent, AppleEvent *theReply, long ref
     if (VIsual_active)
     {
 		end_visual_mode();
+		VIsual_reselect = FALSE;
 		update_curbuf(NOT_VALID);	/* delete the inversion */
     }
-    
+
     fnames = (char_u **) alloc(numFiles * sizeof(char_u *));
-    
+
 /* open each file - keep track of errors	 */
 	for (fileCount = 1; fileCount <= numFiles; fileCount++)
 	{
@@ -2175,16 +2208,16 @@ pascal OSErr HandleODocAE (AppleEvent *theAEvent, AppleEvent *theReply, long ref
 		long		actualSize;
 		AEKeyword	dummyKeyword;
 		DescType	dummyType;
-		
+
 		/* get the alias for the nth file, convert to an FSSpec */
-		error = AEGetNthPtr(&theList, fileCount, typeFSS, &dummyKeyword, &dummyType, 
+		error = AEGetNthPtr(&theList, fileCount, typeFSS, &dummyKeyword, &dummyType,
 								(Ptr) &fileToOpen, sizeof(FSSpec), &actualSize);
 		if (error)
 		{
-            /* TODO: free fnames and it's child */
+			/* TODO: free fnames and it's child */
 			return(error);
 		}
-        GetFullPathFromFSSpec (fname, fileToOpen);
+		GetFullPathFromFSSpec (fname, fileToOpen);
 		fnames[fileCount - 1] = vim_strsave(fname);
 	}
 
@@ -2281,19 +2314,19 @@ mch_display_error()
 		for (p = (char *)error_ga.ga_data; *p; ++p)
 			if (!isspace(*p))
 			{
-                if (STRLEN(p) > 255)
-                    pError[0] = 255;
-                else
-                    pError[0] = STRLEN(p);
+				if (STRLEN(p) > 255)
+					pError[0] = 255;
+				else
+					pError[0] = STRLEN(p);
 
-                STRNCPY(&pError[1], p, pError[0]);
-                ParamText (pError, nil, nil, nil);
-                Alert (128, nil);
+				STRNCPY(&pError[1], p, pError[0]);
+				ParamText (pError, nil, nil, nil);
+				Alert (128, nil);
 				break;
-                /* TODO: handled message longer than 256 chars
-                 *       use auto-sizeable alert
-                 *       or dialog with scrollbars (TextEdit zone)
-                 */
+				/* TODO: handled message longer than 256 chars
+				 *		 use auto-sizeable alert
+				 *		 or dialog with scrollbars (TextEdit zone)
+				 */
 			}
 		ga_clear(&error_ga);
     }
@@ -2303,6 +2336,13 @@ mch_display_error()
  * Get current y mouse coordinate in text window.
  * Return -1 when unknown.
  */
+    int
+gui_mch_get_mouse_x()
+{
+    /* TODO */
+    return -1;
+}
+
     int
 gui_mch_get_mouse_y()
 {
@@ -2317,3 +2357,10 @@ gui_mch_setmouse(x, y)
 {
     /* TODO */
 }
+
+    void
+gui_mch_show_popupmenu(menu)
+	GuiMenu *menu;
+{
+    /* TODO */
+}
