@@ -165,16 +165,36 @@ mytextinit(struct text_info *pTextinfo)
     mywindow(1, 1, Columns, Rows);
 }
 
+    static void
+get_screenbase(void)
+{
+    static union REGS	    regs;
+
+    /* old Hercules grafic card has different base address (Macewicz) */
+    regs.h.ah = 0x0f;
+    (void)int86(0x10, &regs, &regs);	/* int 10 0f */
+    if (regs.h.al == 0x07)		/* video mode 7 -- hercules mono */
+	S_ulScreenBase = 0xb0000;
+    else
+	S_ulScreenBase = 0xb8000;
+}
+
     static void
 mynormvideo(void)
 {
     S_uiAttribute = 0x700;
+    textbackground(0);			/*for delline() etc*/
 }
 
     static void
 mytextattr(int iAttribute)
 {
     S_uiAttribute = (unsigned short)iAttribute << 8;
+    iAttribute >>= 4;
+    if (iAttribute < 8)
+	textbackground(iAttribute);	/*for delline() etc*/
+    else
+	textbackground(0);
 }
 
     static void
@@ -182,6 +202,7 @@ mytextcolor(int iTextColor)
 {
     S_uiAttribute = (unsigned short)((S_uiAttribute & 0xf000)
 					   | (unsigned short)iTextColor << 8);
+    textcolor(iTextColor);		/*for delline() etc*/
 }
 
     static void
@@ -189,6 +210,10 @@ mytextbackground(int iBkgColor)
 {
     S_uiAttribute = (unsigned short)((S_uiAttribute & 0x0f00)
 					 | (unsigned short)(iBkgColor << 12));
+    if (iBkgColor < 8)
+	textbackground(iBkgColor);	/*for delline() etc*/
+    else
+	textbackground(0);
 }
 
 #else
@@ -202,6 +227,65 @@ mytextbackground(int iBkgColor)
 # define mytextbackground textbackground
 #endif
 
+/*
+ * Save/restore the shape of the cursor.
+ * call with FALSE to save, TRUE to restore
+ */
+    static void
+mch_restore_cursor_shape(int restore)
+{
+    static union REGS	    regs;
+    static int		    saved = FALSE;
+
+    if (restore)
+    {
+	if (saved)
+	    regs.h.ah = 0x01;	    /*Set Cursor*/
+	else
+	    return;
+    }
+    else
+    {
+	regs.h.ah = 0x03;	    /*Get Cursor*/
+	regs.h.bh = 0x00;	    /*Page */
+	saved = TRUE;
+    }
+
+    (void)int86(0x10, &regs, &regs);
+}
+
+/*
+ * Set the shape of the cursor.
+ * 'thickness' can be from 0 (thin) to 7 (block)
+ */
+    static void
+mch_set_cursor_shape(int thickness)
+{
+    union REGS	    regs;
+
+    regs.h.ch = 7 - thickness;	    /*Starting Line*/
+    regs.h.cl = 7;		    /*Ending Line*/
+    regs.h.ah = 0x01;		    /*Set Cursor*/
+    (void)int86(0x10, &regs, &regs);
+}
+
+    void
+mch_update_cursor(void)
+{
+    int		idx;
+    int		thickness;
+
+    /*
+     * How the cursor is drawn depends on the current mode.
+     */
+    idx = get_cursor_idx();
+
+    if (cursor_table[idx].shape == SHAPE_BLOCK)
+	thickness = 7;
+    else
+	thickness = (7 * cursor_table[idx].percentage + 90) / 100;
+    mch_set_cursor_shape(thickness);
+}
 
     long_u
 mch_avail_mem(int special)
@@ -217,25 +301,22 @@ mch_avail_mem(int special)
 
 /*
  * Set area where mouse can be moved to: The whole screen.
- * Rows must be valid when calling!
+ * Rows and Columns must be valid when calling!
  */
     static void
 mouse_area(void)
 {
     union REGS	    regs;
-    int		    mouse_y_max;	    /* maximum mouse y coord */
 
     if (mouse_avail)
     {
-	mouse_y_max = Rows * mouse_y_div - 1;
-	if (mouse_y_max < 199)	    /* is this needed? */
-	    mouse_y_max = 199;
 	regs.x.cx = 0;	/* mouse visible between cx and dx */
-	regs.x.dx = 639;
+	regs.x.dx = Columns * mouse_x_div - 1;
 	regs.x.ax = 7;
 	(void)int86(0x33, &regs, &regs);
+
 	regs.x.cx = 0;	/* mouse visible between cx and dx */
-	regs.x.dx = mouse_y_max;
+	regs.x.dx = Rows * mouse_y_div - 1;
 	regs.x.ax = 8;
 	(void)int86(0x33, &regs, &regs);
     }
@@ -268,6 +349,60 @@ show_mouse(int on)
 
 #endif
 
+/*
+ * Version of kbhit() and getch() that use direct console I/O.
+ * This avoids trouble with CTRL-P and the like, and should work over a telnet
+ * connection (it works for Xvi).
+ */
+
+static int cons_key = -1;
+
+/*
+ * Try to get one character directly from the console.
+ * If there is a key, it is stored in cons_key.
+ * Only call when cons_key is -1!
+ */
+    static void
+cons_getkey(void)
+{
+    union REGS regs;
+
+    /* call DOS function 6: Direct console I/O */
+    regs.h.ah = 0x06;
+    regs.h.dl = 0xff;
+    (void)intdos(&regs, &regs);
+    if ((regs.x.flags & 0x40) == 0)	/* zero flag not set? */
+	cons_key = (regs.h.al & 0xff);
+}
+
+/*
+ * Return TRUE if a character is available.
+ */
+    static int
+cons_kbhit(void)
+{
+    if (cons_key < 0)
+	cons_getkey();
+    return (cons_key >= 0);
+}
+
+/*
+ * Return a character from the console.
+ * Should only be called when vim_kbhit() returns TRUE.
+ */
+    static int
+cons_getch(void)
+{
+    int	    c = -1;
+
+    if (cons_key < 0)
+	cons_getkey();
+    c = cons_key;
+    cons_key = -1;
+    return c;
+}
+
+
 #ifdef DJGPP
 /*
  * DJGPP provides a kbhit() function that goes to the BIOS instead of DOS.
@@ -287,8 +422,8 @@ vim_kbhit(void)
 #ifdef kbhit
 # undef kbhit	    /* might have been defined in conio.h */
 #endif
-
 #define kbhit()	vim_kbhit()
+
 #endif
 
 /*
@@ -377,7 +512,8 @@ WaitForChar(long msec)
 	}
 #endif
 
-	if ((p_biosk ? bioskey(1) : kbhit()) || cbrk_pressed
+	if ((p_consk ? cons_kbhit() : p_biosk ? bioskey(1) : kbhit())
+		|| cbrk_pressed
 #ifdef USE_MOUSE
 						    || mouse_click >= 0
 #endif
@@ -621,7 +757,7 @@ mch_inchar(
 #ifdef USE_MOUSE
 	mouse_hidden = TRUE;
 #endif
-	if (p_biosk)
+	if (p_biosk && !p_consk)
 	{
 	    while ((len == 0 || bioskey(1)) && len < maxlen)
 	    {
@@ -658,9 +794,10 @@ mch_inchar(
 	}
 	else
 	{
-	    while ((len == 0 || kbhit()) && len < maxlen)
+	    while ((len == 0 || (p_consk ? cons_kbhit() : kbhit()))
+		    && len < maxlen)
 	    {
-		switch (c = getch())
+		switch (c = (p_consk ? cons_getch() : getch()))
 		{
 		    case 0:
 			/* NUL means that there is another character.
@@ -669,7 +806,10 @@ mch_inchar(
 			 */
 			*buf++ = K_NUL;
 			++len;
-			c = getch();
+			if (p_consk)
+			    c = cons_getch();
+			else
+			    c = getch();
 			break;
 		    case K_NUL:
 			*buf++ = K_NUL;
@@ -833,6 +973,8 @@ mch_windinit(void)
      */
     if (getenv("LFN") == NULL)
 	putenv("LFN=y");
+
+    get_screenbase();
 #endif
 
 #ifdef USE_MOUSE
@@ -856,6 +998,11 @@ mch_windinit(void)
     regs.h.bl = 0x00;
     regs.h.bh = 0x00;
     int86(0x10, &regs, &regs);
+
+    /* Save the old cursor shape */
+    mch_restore_cursor_shape(FALSE);
+    /* Initialise the cursor shape */
+    mch_update_cursor();
 }
 
     int
@@ -867,6 +1014,13 @@ mch_check_win(
     if (*argv[0] != NUL)
 	exe_name = FullName_save((char_u *)argv[0], FALSE);
 
+    /*
+     * Try the DOS search path.  The executable may in
+     * fact be called differently, so try this last.
+     */
+    if (exe_name == NULL || *exe_name == NUL)
+	exe_name = searchpath("vim.exe");
+
     if (isatty(1))
 	return OK;
     return FAIL;
@@ -1005,8 +1159,7 @@ change_drive(int drive)
     intdos(&regs, &regs);   /* get default drive */
     if (regs.h.al == drive - 1)
 	return 0;
-    else
-	return -1;
+    return -1;
 }
 
 /*
@@ -1036,13 +1189,15 @@ mch_FullName(
 	return OK;
     }
 
-#ifdef __BORLANDC__	/* the old Turbo C does not have this */
-    if (_fullpath((char *)buf, (char *)fname, len) == NULL)
+#ifdef __BORLANDC__		/* Only Borland C++ has this */
+    if (_fullpath((char *)buf, (char *)fname, len - 1) == NULL)
     {
 	STRNCPY(buf, fname, len);   /* failed, use the relative path name */
 	slash_adjust(buf);
 	return FAIL;
     }
+    if (mch_isdir(fname))
+	STRCAT(buf, "\\");
     slash_adjust(buf);
     return OK;
 #else			/* almost the same as mch_FullName in os_unix.c */
@@ -1075,7 +1230,13 @@ mch_FullName(
 	    }
 	    else
 	    {
-		q = p + 1;
+		if (p == fname)			/* /fname	    */
+		    q = p + 1;			/* -> /		    */
+		else if (q + 1 == p)		/* ... c:\foo	    */
+		    q = p + 1;			/* -> c:\	    */
+		else				/* but c:\foo\bar   */
+		    q = p;			/* -> c:\foo	    */
+
 		c = *q;			/* truncate at start of fname */
 		*q = NUL;
 #ifdef DJGPP
@@ -1087,7 +1248,11 @@ mch_FullName(
 #endif
 		    retval = FAIL;
 		else
+		{
 		    fname = q;
+		    if (c == '\\')	    /* if we cut the name at a */
+			fname++;	    /* '\', don't add it again */
+		}
 		*q = c;
 	    }
 	}
@@ -1229,6 +1394,7 @@ mch_windexit(int r)
     out_char('\n');
     out_flush();
     ml_close_all(TRUE);		    /* remove all memfiles */
+    mch_restore_cursor_shape(TRUE);
     exit(r);
 }
 
@@ -1260,7 +1426,7 @@ mch_screenmode(char_u *arg)
     int		    mode;
     int		    i;
     static char	   *(names[]) = {"BW40", "C40", "BW80", "C80", "MONO", "C4350"};
-    static int	    modes[]  = { BW40,	 C40,	BW80,	C80,   MONO,   C4350};
+    static int	    modes[]   = { BW40,	  C40,	 BW80,	 C80,	MONO,	C4350};
 
     mode = -1;
     if (isdigit(*arg))		    /* mode number given */
@@ -1280,9 +1446,14 @@ mch_screenmode(char_u *arg)
 	return FAIL;
     }
     textmode(mode);		    /* use Borland function */
+#ifdef DJGPP
+    /* base address may have changed */
+    get_screenbase();
+#endif
 
     /* Screen colors may have changed. */
     out_str(T_ME);
+
 #ifdef USE_MOUSE
     if (mode <= 1 || mode == 4 || mode == 5 || mode == 13 || mode == 0x13)
 	mouse_x_div = 16;
@@ -1416,7 +1587,7 @@ mch_call_shell(
 	/* we use "command" to start the shell, slow but easy */
 	newcmd = alloc(STRLEN(p_sh) + STRLEN(p_shcf) + STRLEN(cmd) + 3);
 	if (newcmd == NULL)
-	    x = 1;
+	    x = -1;
 	else
 	{
 	    sprintf((char *)newcmd, "%s %s %s", p_sh, p_shcf, cmd);
@@ -1443,7 +1614,7 @@ mch_call_shell(
 
     /* resettitle();		    we don't have titles */
     (void)ui_get_winsize();	    /* display mode may have been changed */
-    return (x ? FAIL : OK);
+    return x;
 }
 
 /*
@@ -1492,12 +1663,16 @@ namelowcpy(
 
 /*
  * Recursive function to expand one path section with wildcards.
+ * This only expands '?' and '*'.
  * Return the number of matches found.
+ * "path" has backslashes before chars that are not to be expanded, starting
+ * at "wildc".
  */
-    int
-mch_expandpath(
+    static int
+dos_expandpath(
     struct growarray	*gap,
     char_u		*path,
+    char_u		*wildc,
     int			flags)
 {
     char_u		*buf;
@@ -1505,6 +1680,7 @@ mch_expandpath(
     int			start_len, c;
     struct ffblk	fb;
     int			matches;
+    int			len;
 
     start_len = gap->ga_len;
     buf = alloc(STRLEN(path) + BASENAMELEN + 5);   /* make room for file name */
@@ -1520,21 +1696,21 @@ mch_expandpath(
     e = NULL;
     while (*path)
     {
-	if (*path == '\\' || *path == ':' || *path == '/')
+	if (path >= wildc && rem_backslash(path))	/* remove a backslash */
+	    ++path;
+	else if (*path == '\\' || *path == ':' || *path == '/')
 	{
-	    if (e)
+	    if (e != NULL)
 		break;
 	    else
-		s = p;
+		s = p + 1;
 	}
-	if (*path == '*' || *path == '?')
+	else if (*path == '*' || *path == '?')
 	    e = p;
 	*p++ = *path++;
     }
     e = p;
-    if (s)
-	s++;
-    else
+    if (s == NULL)
 	s = buf;
 
     /* if the file name ends in "*" and does not contain a ".", addd ".*" */
@@ -1561,11 +1737,22 @@ mch_expandpath(
 	/* ignore "." and ".." */
 	if (*s != '.' || (s[1] != NUL && (s[1] != '.' || s[2] != NUL)))
 	{
-	    STRCAT(buf, path);
+	    len = STRLEN(buf);
+	    STRCPY(buf + len, path);
 	    if (mch_has_wildcard(path))
-		(void)mch_expandpath(gap, buf, flags);
-	    else if (mch_getperm(buf) >= 0)	/* add existing file */
-		addfile(gap, buf, flags);
+	    {
+		/* need to expand another component of the path */
+		/* remove backslashes for the remaining components only */
+		(void)dos_expandpath(gap, buf, buf + len + 1, flags);
+	    }
+	    else
+	    {
+		/* no more wildcards, check if there is a match */
+		/* remove backslashes for the remaining components only */
+		backslash_halve(buf + len + 1, FALSE);
+		if (mch_getperm(buf) >= 0)	/* add existing file */
+		    addfile(gap, buf, flags);
+	    }
 	}
 	c = findnext(&fb);
     }
@@ -1578,6 +1765,15 @@ mch_expandpath(
     return matches;
 }
 
+    int
+mch_expandpath(
+    struct growarray	*gap,
+    char_u		*path,
+    int			flags)
+{
+    return dos_expandpath(gap, path, path, flags);
+}
+
 /*
  * The normal chdir() does not change the default drive.
  * This one does.
@@ -1665,12 +1861,26 @@ mch_rename(const char *OldFile, const char *NewFile)
 mch_getenv(char_u *var)
 {
     char_u  *retval;
+    char_u  *var_copy, *p;
 
-    retval = (char_u *)getenv((char *)var);
+    /*
+     * Take a copy of the argument, and force it to upper case before passing
+     * to getenv().  On DOS systems, getenv() doesn't like lower-case argument
+     * (unlike Win32 et al.)
+     */
+    var_copy = vim_strsave(var);
+    if (var_copy == NULL)
+	return NULL;
+    for (p = var_copy; *p != NUL; p++)
+	*p = toupper(*p);
 
-    if (retval == NULL && STRCMP(var, "VIM") == 0)
+    retval = (char_u *)getenv((char *)var_copy);
+
+    if (retval == NULL && STRCMP(var_copy, "VIM") == 0)
 	retval = (char_u *)getenv("HOME");
 
+    vim_free(var_copy);
+
     return retval;
 }
 
