@@ -22,7 +22,7 @@
 #define EFN	0			/* Event flag */
 
 #ifdef PROTO
-# define DESC int
+# typedef int DESC
 #endif
 
 static void	vms_flushbuf(void);
@@ -421,7 +421,7 @@ mch_suspend(void)
 #endif
 #ifdef SIGTSTP
     vms_flushbuf();	/* needed to make cursor visible on some systems */
-    settmode(0);
+    settmode(TMODE_COOK);
     vms_flushbuf();	/* needed to disable mouse on some systems */
     kill(0, SIGTSTP);	/* send ourselves a STOP signal */
 
@@ -433,19 +433,19 @@ mch_suspend(void)
 	vim_free(oldtitle);
 	oldtitle = NULL;
     }
-    settmode(1);
+    settmode(TMODE_RAW);
 #else
     if (!sw_detached)
     {
-	OUT_STR("new shell started\n");
-	(void)mch_call_shell(NULL, SHELL_COOKED);
+	suspend_shell();
 	return;
     }
     vms_flushbuf();
-    settmode(0);				/* set to cooked mode */
+    settmode(TMODE_COOK);		/* set to cooked mode */
 
     sprintf(symstr, "%08X", pid);
-    mch_setenv("VIMPID", symstr);
+    /* added 3rd parameter SK 980904 */
+    mch_setenv("VIMPID", symstr, 1);
 
     /* windgoto(wins[cw].rows-1,0);	*/
     /* putchar('\r');	*/
@@ -456,15 +456,15 @@ mch_suspend(void)
     res = lib$attach(&ppid);
     if (!(res & 1))
     {
-	OUT_STR("\nDetach failed, new shell started\n");
-	(void)mch_call_shell(NULL, SHELL_COOKED);
+	OUT_STR("\nDetach failed");
+	suspend_shell();
     }
 
     /* set_tty(P(P_LI), P(P_CO));*/
     /* screenclear(cw);*/
     /* updatescreen();*/
 
-    settmode(1);					/* set to raw mode */
+    settmode(TMODE_RAW);		/* set to raw mode */
     resettitle();
 #endif
     need_check_timestamps = TRUE;
@@ -585,7 +585,8 @@ get_x11_windis(void)
 	    x11_display = NULL;
 	    x11_display_opened_here = FALSE;
 	}
-	return gui_get_x11_windis(&x11_window, &x11_display);
+	gui_get_x11_windis(&x11_window, &x11_display);
+	return OK;
     }
 #endif
 
@@ -1133,7 +1134,7 @@ mch_isdir(char_u *name)
 mch_windexit(int r)
 {
     mch_settitle(oldtitle, oldicon);	/* restore xterm title */
-    settmode(0);
+    settmode(TMODE_COOK);
     exiting = TRUE;
     stoptermcap();
     vms_flushbuf();
@@ -1425,8 +1426,7 @@ mch_set_winsize(void)
 /*
  *	options		SHELL_FILTER	if called by do_filter()
  *			SHELL_COOKED	if term needs cooked mode
- *			SHELL_EXPAND	if called by
- *					mch_expand_wildcards()
+ *			SHELL_EXPAND	if called by mch_expand_wildcards()
  */
 
     int
@@ -1437,12 +1437,13 @@ mch_call_shell(char_u *cmd, int options)
 
     vms_flushbuf();
     if (options & SHELL_COOKED)
-	settmode(0);				/* set to cooked mode */
+	settmode(TMODE_COOK);			/* set to cooked mode */
     if (cmd)
     {
 	if (ofn = strchr((char *)cmd, '>'))
 	    *ofn++ = '\0';
-	if (ifn = strchr((char *)cmd, '<')) {
+	if (ifn = strchr((char *)cmd, '<'))
+	{
 	   char *p;
 	    *ifn++ = '\0';
 	    /* chop off any trailing spaces */
@@ -1465,9 +1466,9 @@ mch_call_shell(char_u *cmd, int options)
 	msg_outnum((long)x);
 	OUT_STR(" returned\n");
     }
-    settmode(1);					/* set to raw mode */
+    settmode(TMODE_RAW);			/* set to raw mode */
     resettitle();
-    return(x ? FAIL : OK);
+    return x;
 }
 
 #ifdef TESTING_PTY
@@ -1511,9 +1512,11 @@ mch_breakcheck(void)
     int
 WaitForChar(long msec)
 {
-    if (vim_is_input_buf_empty()) {
-	if(RealWaitForChar(0,msec)) {
-	    add_to_input_buf((char_u *)ibuf,1);
+    if (vim_is_input_buf_empty())
+    {
+	if (RealWaitForChar(0, msec))
+	{
+	    add_to_input_buf((char_u *)ibuf, 1);
 	    return 1;
 	}
 	else
@@ -1558,8 +1561,8 @@ char_u **vms_fmatch = NULL;
  * We want to save each match for later retrieval.
  *
  * Returns:  1 - continue finding matches
- *           0 - stop trying to find any further mathces
- * 
+ *	     0 - stop trying to find any further mathces
+ *
  */
 int
 vms_wproc( char *name, int type )
@@ -1617,11 +1620,11 @@ vms_wproc( char *name, int type )
 	vms_fmatch[vms_match_num] = vim_strsave((char_u *)xname);
     }
     else {
-	vms_fmatch[vms_match_num] = 
+	vms_fmatch[vms_match_num] =
 	    vim_strsave((char_u *)xname);
     }
 #else
-    vms_fmatch[vms_match_num] = 
+    vms_fmatch[vms_match_num] =
 	vim_strsave((char_u *)xname);
 #endif
     ++vms_match_num;
@@ -1656,11 +1659,13 @@ mch_expand_wildcards(int num_pat, char_u **pat, int *num_file, char_u ***file, i
     files_alloced = EXPL_ALLOC_INC;
     files_free = EXPL_ALLOC_INC;
     *file = (char_u **) alloc(sizeof(char_u **) * files_alloced);
-    if (*file == NULL ){
+    if (*file == NULL )
+    {
 	*num_file = 0;
 	return FAIL;
     }
-    for (i=0; i<num_pat; i++) {
+    for (i = 0; i < num_pat; i++)
+    {
 	/* expand environment var or home dir */
 	if (vim_strchr(pat[i],'$') || vim_strchr(pat[i],'~'))
 	    expand_env(pat[i],buf,MAXPATHL);
@@ -1685,7 +1690,7 @@ mch_expand_wildcards(int num_pat, char_u **pat, int *num_file, char_u ***file, i
 	    /* allocate memory for pointers */
 	    if(--files_free < 1 ) {
 		files_alloced += EXPL_ALLOC_INC;
-		*file = (char_u **)realloc(*file, 
+		*file = (char_u **)realloc(*file,
 		    sizeof(char_u **) * files_alloced);
 		if( *file == NULL ) {
 		    *file = (char_u **)"";
@@ -1867,7 +1872,7 @@ vms_sys(char *cmd, char *log, char *inp)
     if (inp)
 	vul_desc(&idsc, inp);
 
-    status = lib$spawn(cmd ? &cdsc : NULL, inp ? &idsc : NULL, 
+    status = lib$spawn(cmd ? &cdsc : NULL, inp ? &idsc : NULL,
 	    log ? &ldsc : NULL, 0, 0, 0, &substatus, 0, 0, 0);
     if (status != SS$_NORMAL)
 	substatus = status;
@@ -1938,14 +1943,13 @@ mch_expandpath(struct growarray *gap, char_u *path, int flags)
     char	*cp;
 
     vms_match_num = 0;
-    cnt = decc$to_vms((char *)path,vms_wproc,1,0);
-    if(cnt > 0) cnt = vms_match_num;
-    for(i=0;i<cnt;i++) {
+    cnt = decc$to_vms((char *)path, vms_wproc, 1, 0);
+    if (cnt > 0)
+	cnt = vms_match_num;
+    for (i = 0; i < cnt; i++)
+    {
 	if (mch_getperm(vms_fmatch[i]) >= 0) /* add existing file */
-	{
 	    addfile(gap, vms_fmatch[i], flags);
-	}
-
     }
     return cnt;
 }
