@@ -71,6 +71,7 @@ FILE* fdDump = NULL;
 # define CONSOLE_CURSOR_INFO int
 # define LPCSTR char_u *
 # define WINBASEAPI
+# define INPUT_RECORD int
 #endif
 
 #ifndef USE_GUI_WIN32
@@ -104,14 +105,14 @@ char * _fullpath(char *buf, char *fname, int len)
 {
     LPTSTR toss;
 
-   return (char *) GetFullPathName(fname, len, buf, &toss);
+    return (char *)GetFullPathName(fname, len, buf, &toss);
 }
 
 int _chdrive(int drive)
 {
     char temp [3] = "-:";
-    temp[0] = drive + 'A';
-    return SetCurrentDirectory(temp);
+    temp[0] = drive + 'A' - 1;
+    return !SetCurrentDirectory(temp);
 }
 #else
 #ifdef __BORLANDC__
@@ -177,37 +178,57 @@ static void cursor_visible(BOOL fVisible);
 static BOOL write_chars(LPCSTR pchBuf, DWORD cchToWrite, DWORD* pcchWritten);
 static char_u tgetch(void);
 static void create_conin(void);
-
+static void mch_set_cursor_shape(int thickness);
+static int s_cursor_visible = TRUE;
 static int did_create_conin = FALSE;
+#else
+static int s_dont_use_vimrun = TRUE;
+static char *vimrun_path = "vimrun ";
 #endif
 
+/*
+ * The old solution for opening a console is still there.
+ */
+//#define OLD_CONSOLE_STUFF
+
+#ifdef OLD_CONSOLE_STUFF
 static void mch_open_console(void);
 static void mch_close_console(int wait_key, DWORD ret);
+#endif
 struct growarray error_ga = {0, 0, 0, 0, NULL};
 
+#ifndef USE_GUI_WIN32
+static int suppress_winsize = 1;	/* don't fiddle with console */
+#endif
+
 /* This symbol is not defined in older versions of the SDK or Visual C++ */
 
 #ifndef VER_PLATFORM_WIN32_WINDOWS
 # define VER_PLATFORM_WIN32_WINDOWS 1
 #endif
 
-static DWORD PlatformId(void);
+static void PlatformId(void);
 static DWORD g_PlatformId;
 
 /*
- *  Returns VER_PLATFORM_WIN32_NT (NT) or VER_PLATFORM_WIN32_WINDOWS (Win95)
+ * Set g_PlatformId to VER_PLATFORM_WIN32_NT (NT) or
+ * VER_PLATFORM_WIN32_WINDOWS (Win95).
  */
-    static DWORD
+    static void
 PlatformId(void)
 {
-    OSVERSIONINFO ovi;
+    static int done = FALSE;
 
-    ovi.dwOSVersionInfoSize = sizeof(ovi);
-    GetVersionEx(&ovi);
+    if (!done)
+    {
+	OSVERSIONINFO ovi;
 
-    g_PlatformId = ovi.dwPlatformId;
+	ovi.dwOSVersionInfoSize = sizeof(ovi);
+	GetVersionEx(&ovi);
 
-    return g_PlatformId;
+	g_PlatformId = ovi.dwPlatformId;
+	done = TRUE;
+    }
 }
 
 /*
@@ -312,6 +333,9 @@ const static struct
 
 /* The return code indicates key code size. */
     static int
+#ifdef __BORLANDC__
+    __stdcall
+#endif
 win32_kbd_patch_key(
     KEY_EVENT_RECORD* pker)
 {
@@ -788,6 +812,51 @@ decode_mouse_event(
 
 
 #ifndef USE_GUI_WIN32	    /* this isn't used for the GUI */
+/*
+ * Set the shape of the cursor.
+ * 'thickness' can be from 1 (thin) to 99 (block)
+ */
+    static void
+mch_set_cursor_shape(int thickness)
+{
+    CONSOLE_CURSOR_INFO ConsoleCursorInfo;
+    ConsoleCursorInfo.dwSize = thickness;
+    ConsoleCursorInfo.bVisible = s_cursor_visible;
+
+    SetConsoleCursorInfo(g_hCurOut, &ConsoleCursorInfo);
+}
+
+    void
+mch_update_cursor(void)
+{
+    int		idx;
+    int		thickness;
+
+    /*
+     * How the cursor is drawn depends on the current mode.
+     */
+    idx = get_cursor_idx();
+
+    if (cursor_table[idx].shape == SHAPE_BLOCK)
+	thickness = 99;	/* 100 doesn't work on W95 */
+    else
+	thickness = cursor_table[idx].percentage;
+    mch_set_cursor_shape(thickness);
+}
+
+/*
+ * Handle FOCUS_EVENT.
+ */
+    static void
+handle_focus_event(INPUT_RECORD ir)
+{
+    g_fJustGotFocus = ir.Event.FocusEvent.bSetFocus;
+    if (g_fJustGotFocus)
+	check_timestamps();
+    apply_autocmds(g_fJustGotFocus ? EVENT_FOCUSGAINED
+				: EVENT_FOCUSLOST, NULL, NULL, FALSE, curbuf);
+}
+
 /*
  * Wait until console input from keyboard or mouse is available,
  * or the time is up.
@@ -832,8 +901,8 @@ WaitForChar(long msec)
 
 	if (cRecords > 0)
 	{
-#ifdef MULTI_BYTE
-	    /* Windows IME sends two '\n's with only one 'ENTER'. 
+#ifdef MULTI_BYTE_IME
+	    /* Windows IME sends two '\n's with only one 'ENTER'.
 	       first, wVirtualKeyCode == 13. second, wVirtualKeyCode == 0 */
 	    if (ir.EventType == KEY_EVENT && ir.Event.KeyEvent.bKeyDown)
 	    {
@@ -852,10 +921,9 @@ WaitForChar(long msec)
 #endif
 
 	    ReadConsoleInput(g_hConIn, &ir, 1, &cRecords);
-#ifdef USE_CLIPBOARD
+
 	    if (ir.EventType == FOCUS_EVENT)
-		g_fJustGotFocus = ir.Event.FocusEvent.bSetFocus;
-#endif
+		handle_focus_event(ir);
 	    else if (ir.EventType == WINDOW_BUFFER_SIZE_EVENT)
 		set_winsize(Rows, Columns, FALSE);
 #ifdef USE_MOUSE
@@ -917,13 +985,8 @@ tgetch(void)
 	    if (decode_key_event(&ir.Event.KeyEvent, &ch, &g_chPending, TRUE))
 		return ch;
 	}
-#ifdef USE_CLIPBOARD
 	else if (ir.EventType == FOCUS_EVENT)
-	{
-	    g_fJustGotFocus = ir.Event.FocusEvent.bSetFocus;
-	    /* TRACE("tgetch: Focus %d\n", g_fJustGotFocus); */
-	}
-#endif
+	    handle_focus_event(ir);
 	else if (ir.EventType == WINDOW_BUFFER_SIZE_EVENT)
 	{
 	    set_winsize(Rows, Columns, FALSE);
@@ -1039,6 +1102,34 @@ mch_inchar(
 }
 
 #ifdef USE_GUI_WIN32
+
+#include <shellapi.h>	/* required for FindExecutable() */
+
+    static int
+executable_exists(char *name)
+{
+    char vimrun_location[2 * _MAX_PATH + 2];
+    char widename[2 * _MAX_PATH];
+
+    /* There appears to be a bug in FindExecutableA() on Windows NT.
+     * Use FindExecutableW() instead... */
+    if (g_PlatformId == VER_PLATFORM_WIN32_NT)
+    {
+	MultiByteToWideChar(CP_ACP, 0, (LPCTSTR)name, -1,
+						 (LPWSTR)widename, _MAX_PATH);
+	if (FindExecutableW((LPCWSTR)widename, (LPCWSTR)"",
+				     (LPWSTR)vimrun_location) > (HINSTANCE)32)
+	    return TRUE;
+    }
+    else
+    {
+	if (FindExecutableA((LPCTSTR)name, (LPCTSTR)"",
+				     (LPTSTR)vimrun_location) > (HINSTANCE)32)
+	    return TRUE;
+    }
+    return FALSE;
+}
+
 /*
  * GUI version of mch_windinit().
  */
@@ -1049,11 +1140,48 @@ mch_windinit()
 
     PlatformId();
 
+    /* Let critical errors result in a failure, not in a dialog box.  Required
+     * for the timestamp test to work on removed floppies. */
+    SetErrorMode(SEM_FAILCRITICALERRORS);
+
+    _fmode = O_BINARY;		/* we do our own CR-LF translation */
+
     /* Specify window size.  Is there a place to get the default from? */
     Rows = 25;
     Columns = 80;
 
-    _fmode = O_BINARY;		/* we do our own CR-LF translation */
+    /* Look for 'vimrun' */
+    if (!gui_is_win32s())
+    {
+	char_u vimrun_location[_MAX_PATH + 2];
+
+	/* First try in same directory as gvim.exe */
+	STRCPY(vimrun_location, exe_name);
+	STRCPY(gettail(vimrun_location), "vimrun.exe");
+	if (mch_getperm(vimrun_location) >= 0)
+	{
+	    STRCPY(gettail(vimrun_location), "vimrun ");
+	    vimrun_path = (char *)vim_strsave(vimrun_location);
+	    s_dont_use_vimrun = FALSE;
+	}
+	else if (executable_exists("vimrun.exe"))
+	    s_dont_use_vimrun = FALSE;
+
+	if (s_dont_use_vimrun)
+	    MessageBox(NULL,
+			"VIMRUN.EXE not found in your $PATH.\n"
+			"External commands will not pause after completion.\n"
+			"See  :help win32-vimrun  for more information.",
+			"Vim Warning",
+			MB_ICONWARNING);
+    }
+
+    /*
+     * If "finstr.exe" doesn't exist, use "grep -n" for 'grepprg'.
+     * Otherwise the default "findstr /n" is used.
+     */
+    if (!executable_exists("findstr.exe"))
+	set_option_value((char_u *)"grepprg", 0, (char_u *)"grep -n");
 
 #ifdef USE_CLIPBOARD
     clip_init(TRUE);
@@ -1114,6 +1242,10 @@ mch_windinit()
 
     PlatformId();
 
+    /* Let critical errors result in a failure, not in a dialog box.  Required
+     * for the timestamp test to work on removed floppies. */
+    SetErrorMode(SEM_FAILCRITICALERRORS);
+
     _fmode = O_BINARY;		/* we do our own CR-LF translation */
     out_flush();
 
@@ -1503,15 +1635,19 @@ mch_FullName(
     int len,
     int force)
 {
-    int nResult = FAIL;
+    int		nResult = FAIL;
 
     if (fname == NULL)		/* always fail */
 	return FAIL;
 
-    if (_fullpath(buf, fname, len) == NULL)
+    if (_fullpath(buf, fname, len - 1) == NULL)
 	STRNCPY(buf, fname, len);   /* failed, use the relative path name */
     else
+    {
+	if (mch_isdir(fname))
+	    STRCAT(buf, "\\");
 	nResult = OK;
+    }
 
     fname_case(buf);
 
@@ -1528,6 +1664,10 @@ mch_isFullName(
 {
     char szName[_MAX_PATH+1];
 
+    /* A name like "d:/foo" is always absolute */
+    if (fname[0] && fname[1] == ':' && (fname[2] == '/' || fname[2] == '\\'))
+	return TRUE;
+
     mch_FullName(fname, szName, _MAX_PATH, FALSE);
 
     return strcoll(fname, szName) == 0;
@@ -1801,17 +1941,26 @@ ResizeConBufAndWindow(
     srWindowRect.Right =  (SHORT) (min(xSize, coordScreen.X) - 1);
     srWindowRect.Bottom = (SHORT) (min(ySize, coordScreen.Y) - 1);
 
-    /* define the new console buffer size */
-    coordScreen.X = xSize;
-    coordScreen.Y = ySize;
-
     if (GetConsoleScreenBufferInfo(g_hCurOut, &csbi))
     {
-	if ((csbi.srWindow.Bottom - csbi.srWindow.Top + 1) < ySize)
+	int sx, sy;
+
+	sx = csbi.srWindow.Right - csbi.srWindow.Left + 1;
+	sy = csbi.srWindow.Bottom - csbi.srWindow.Top + 1;
+	if (sy < ySize || sx < xSize)
 	{
 	    /*
-	     * increasing number of lines, do buffer first
+	     * Increasing number of lines/columns, do buffer first.
+	     * Use the maximal size in x and y direction.
 	     */
+	    if (sy < ySize)
+		coordScreen.Y = ySize;
+	    else
+		coordScreen.Y = sy;
+	    if (sx < xSize)
+		coordScreen.X = xSize;
+	    else
+		coordScreen.X = sx;
 	    SetConsoleScreenBufferSize(hConsole, coordScreen);
 	}
     }
@@ -1828,6 +1977,10 @@ ResizeConBufAndWindow(
 #endif
     }
 
+    /* define the new console buffer size */
+    coordScreen.X = xSize;
+    coordScreen.Y = ySize;
+
     if (!SetConsoleScreenBufferSize(hConsole, coordScreen))
     {
 #ifdef MCH_WRITE_DUMP
@@ -1851,7 +2004,16 @@ ResizeConBufAndWindow(
     void
 mch_set_winsize()
 {
-    COORD coordScreen = GetLargestConsoleWindowSize(g_hCurOut);
+    COORD coordScreen;
+
+    /* Don't change window size while still starting up */
+    if (suppress_winsize)
+    {
+	suppress_winsize = 2;
+	return;
+    }
+
+    coordScreen = GetLargestConsoleWindowSize(g_hCurOut);
 
     /* Clamp Rows and Columns to reasonable values */
     if (Rows > coordScreen.Y)
@@ -1863,6 +2025,20 @@ mch_set_winsize()
     set_scroll_region(0, 0, Columns - 1, Rows - 1);
 }
 
+/*
+ * Called when started up, to set the winsize that was delayed.
+ */
+    void
+mch_set_winsize_now()
+{
+    if (suppress_winsize == 2)
+    {
+	suppress_winsize = 0;
+	mch_set_winsize();
+	check_winsize();	    /* in case 'columns' changed */
+    }
+    suppress_winsize = 0;
+}
 #endif /* USE_GUI_WIN32 */
 
 
@@ -1876,6 +2052,8 @@ mch_suspend()
 }
 
 #if defined(USE_GUI_WIN32) || defined(PROTO)
+
+#ifdef OLD_CONSOLE_STUFF
 /*
  * Functions to open and close a console window.
  * The open function sets the size of the window to 25x80, to avoid problems
@@ -1953,6 +2131,7 @@ mch_close_console(int wait_key, DWORD ret)
     /* Close the console window */
     FreeConsole();
 }
+#endif
 
 #ifdef mch_errmsg
 # undef mch_errmsg
@@ -2014,7 +2193,7 @@ mch_display_error()
 static BOOL fUseConsole = TRUE;
 
     static int
-mch_system(char * cmd, int options)
+mch_system(char *cmd, int options)
 {
     STARTUPINFO		si;
     PROCESS_INFORMATION pi;
@@ -2024,10 +2203,19 @@ mch_system(char * cmd, int options)
     si.lpReserved = NULL;
     si.lpDesktop = NULL;
     si.lpTitle = NULL;
-    si.dwFlags = 0;
+    si.dwFlags = STARTF_USESHOWWINDOW;
+    /*
+     * It's nicer to run a filter command in a minimized window, but in
+     * Windows 95 this makes the command MUCH slower.
+     */
+    if ((options & SHELL_DOOUT) && !mch_windows95())
+	si.wShowWindow = SW_SHOWMINIMIZED;
+    else
+	si.wShowWindow = SW_SHOWNORMAL;
     si.cbReserved2 = 0;
     si.lpReserved2 = NULL;
 
+#ifdef OLD_CONSOLE_STUFF
     /* Create a console for the process. This lets us write a termination
      * message at the end, and wait for the user to close the console
      * window manually...
@@ -2048,18 +2236,31 @@ mch_system(char * cmd, int options)
 	    WriteConsole(hStderr, "\n", 1, &number, NULL);
 	}
     }
+#else
+    /* There is a strange error on Windows 95 when using "c:\\command.com".
+     * When the "c:\\" is left out it works OK...? */
+    if (mch_windows95()
+	    && (STRNICMP(cmd, "c:/command.com", 14) == 0
+		|| STRNICMP(cmd, "c:\\command.com", 14) == 0))
+	cmd += 3;
+#endif
 
     /* Now, run the command */
-    CreateProcess (NULL,		/* Executable name */
-		   cmd,			/* Command to execute */
-		   NULL,		/* Process security attributes */
-		   NULL,		/* Thread security attributes */
-		   FALSE,		/* Inherit handles */
-		   0,			/* Creation flags */
-		   NULL,		/* Environment */
-		   NULL,		/* Current directory */
-		   &si,			/* Startup information */
-		   &pi);		/* Process information */
+    CreateProcess(NULL,			/* Executable name */
+		  cmd,			/* Command to execute */
+		  NULL,			/* Process security attributes */
+		  NULL,			/* Thread security attributes */
+		  FALSE,		/* Inherit handles */
+		  CREATE_DEFAULT_ERROR_MODE |	/* Creation flags */
+#ifdef OLD_CONSOLE_STUFF
+		  0,
+#else
+		  CREATE_NEW_CONSOLE,
+#endif
+		  NULL,			/* Environment */
+		  NULL,			/* Current directory */
+		  &si,			/* Startup information */
+		  &pi);			/* Process information */
 
 
     /* Wait for the command to terminate before continuing */
@@ -2098,8 +2299,10 @@ mch_system(char * cmd, int options)
     /* Close the console window. If we are not redirecting output, wait for
      * the user to press a key.
      */
+#ifdef OLD_CONSOLE_STUFF
     if (fUseConsole)
 	mch_close_console(!(options & SHELL_DOOUT), ret);
+#endif
 
     return ret;
 }
@@ -2129,7 +2332,8 @@ mch_call_shell(
     /*
      * ALWAYS switch to non-termcap mode, otherwise ":r !ls" may crash.
      */
-    if (g_hCurOut == g_hConOut && STRNICMP(cmd, "start ", 6) != 0)
+    if (g_hCurOut == g_hConOut &&
+	    ((cmd == NULL) || STRNICMP(cmd, "start ", 6) != 0))
     {
 	termcap_mode_end();
 	stopped_termcap_mode = TRUE;
@@ -2165,15 +2369,18 @@ mch_call_shell(
 
     if (cmd == NULL)
     {
-	/* Set the SHELL_DOOUT flag, to avoid the "hit any key" prompt */
-	x = mch_system(p_sh, options | SHELL_DOOUT);
+	x = mch_system(p_sh, options);
     }
     else
     {
 	/* we use "command" or "cmd" to start the shell; slow but easy */
 	char_u *newcmd;
 
-	newcmd = lalloc(STRLEN(p_sh) + STRLEN(p_shcf) + STRLEN(cmd) + 3, TRUE);
+	newcmd = lalloc(
+#ifdef USE_GUI_WIN32
+		STRLEN(vimrun_path) +
+#endif
+		STRLEN(p_sh) + STRLEN(p_shcf) + STRLEN(cmd) + 10, TRUE);
 	if (newcmd != NULL)
 	{
 	    if (STRNICMP(cmd, "start ", 6) == 0)
@@ -2206,14 +2413,26 @@ mch_call_shell(
 			&pi))			// Process information
 		    x = 0;
 		else
-		    x = 1;
+		    x = -1;
 		/* Close the handles to the subprocess, so that it goes away */
 		CloseHandle(pi.hThread);
 		CloseHandle(pi.hProcess);
 	    }
 	    else
 	    {
-		sprintf((char *)newcmd, "%s %s %s", p_sh, p_shcf, cmd);
+		sprintf((char *)newcmd, "%s%s %s %s",
+#if defined(OLD_CONSOLE_STUFF) || !defined(USE_GUI_WIN32)
+			"",
+#else
+			/*
+			 * Do we need to use "vimrun"?
+			 */
+			((options & SHELL_DOOUT) || s_dont_use_vimrun) ?
+			    "" : vimrun_path,
+#endif
+			p_sh,
+			p_shcf,
+			cmd);
 		x = mch_system((char *)newcmd, options);
 	    }
 	    vim_free(newcmd);
@@ -2250,7 +2469,7 @@ mch_call_shell(
 	termcap_mode_start();
 #endif
 
-    return (x ? FAIL : OK);
+    return x;
 }
 
 
@@ -2284,10 +2503,11 @@ pstrcmp(
  * Recursively build up a list of files in "gap" matching the first wildcard
  * in `path'.  Called by expand_wildcards().
  */
-    int
-mch_expandpath(
+    static int
+win32_expandpath(
     struct growarray	*gap,
     char_u		*path,
+    char_u		*wildc,
     int			flags)
 {
     char		buf[_MAX_PATH+1];
@@ -2297,6 +2517,7 @@ mch_expandpath(
     HANDLE		hFind;
     int			matches;
     int			start_dot_ok;
+    int			len;
 
     start_len = gap->ga_len;
 
@@ -2309,21 +2530,21 @@ mch_expandpath(
     e = NULL;
     while (*path)
     {
-	if (*path == '\\' || *path == ':' || *path == '/')
+	if (path >= wildc && rem_backslash(path))	/* remove a backslash */
+	    ++path;
+	else if (*path == '\\' || *path == ':' || *path == '/')
 	{
 	    if (e)
 		break;
 	    else
-		s = p;
+		s = p + 1;
 	}
-	if (*path == '*' || *path == '?')
+	else if (*path == '*' || *path == '?')
 	    e = p;
 	*p++ = *path++;
     }
     e = p;
-    if (s)
-	s++;
-    else
+    if (s == NULL)
 	s = buf;
 
 #ifdef USE_GUI_WIN32
@@ -2366,11 +2587,22 @@ mch_expandpath(
 			    && (s[1] != '.' || s[2] != NUL)))
 		    && (*path == NUL || mch_isdir(buf)))
 	    {
-		STRCAT(buf, path);
+		len = STRLEN(buf);
+		STRCPY(buf + len, path);
 		if (mch_has_wildcard(path))	    /* handle more wildcards */
-		    (void)mch_expandpath(gap, buf, flags);
+		{
+		    /* need to expand another component of the path */
+		    /* remove backslashes for the remaining components only */
+		    (void)win32_expandpath(gap, buf, buf + len + 1, flags);
+		}
 		else if (mch_getperm(buf) >= 0)	    /* add existing file */
-		    addfile(gap, buf, flags);
+		{
+		    /* no more wildcards, check if there is a match */
+		    /* remove backslashes for the remaining components only */
+		    backslash_halve(buf + len + 1, FALSE);
+		    if (mch_getperm(buf) >= 0)	/* add existing file */
+			addfile(gap, buf, flags);
+		}
 	    }
 
 	    c = FindNextFile(hFind, &fb);
@@ -2384,6 +2616,14 @@ mch_expandpath(
     return matches;
 }
 
+    int
+mch_expandpath(
+    struct growarray	*gap,
+    char_u		*path,
+    int			flags)
+{
+    return win32_expandpath(gap, path, path, flags);
+}
 
 /*
  * The normal _chdir() does not change the default drive.  This one does.
@@ -2537,6 +2777,12 @@ termcap_mode_end()
     g_attrSave = g_attrCurrent;
 
     ResizeConBufAndWindow(g_hCurOut, g_nOldColumns, g_nOldRows);
+
+    /* This weird Sleep(0) is required to allow Windows to really resize the
+     * console window.  Apparently it's done asynchronously, which may cause
+     * the following screenbuffer switch to go wrong */
+    Sleep(0);
+
     check_winsize();
 
     g_hCurOut = g_hSavOut;
@@ -2903,12 +3149,8 @@ visual_bell()
 cursor_visible(
     BOOL fVisible)
 {
-    CONSOLE_CURSOR_INFO cci;
-
-    cci.bVisible = fVisible;
-    /* make cursor big and visible (100 on Win95 makes it disappear)  */
-    cci.dwSize = 99;	       /* 100 percent cursor */
-    SetConsoleCursorInfo(g_hCurOut, &cci);
+    s_cursor_visible = fVisible;
+    mch_update_cursor();
 }
 
 
